@@ -1,10 +1,8 @@
 reqMktData <-
 function (conn, Contract, tickGenerics='100,101,104,106,165,221,225,236',
-          snapshot = FALSE, tickerId = "1", timeStamp=TRUE,playback=1,
+          snapshot = FALSE, tickerId = "1", timeStamp="%Y%m%d %H:%M:%OS",playback=1,
           file='', verbose=TRUE,
-          eventTickPrice,eventTickSize,
-          eventTickOption,eventTickGeneric,
-          eventTickString,eventTickEFP,CALLBACK,...) 
+          eventWrapper=eWrapper(), CALLBACK=twsCALLBACK, ...)
 {
     if (!inherits(conn,"twsConnection") )
         stop("tws connection object required")
@@ -16,7 +14,7 @@ function (conn, Contract, tickGenerics='100,101,104,106,165,221,225,236',
         if (class(Contract[[n]]) != "twsContract") 
             stop("twsContract required")
       }
-    }
+    } #else file <- ""
 
     con <- conn[[1]]
     if (!isOpen(con)) 
@@ -37,36 +35,8 @@ function (conn, Contract, tickGenerics='100,101,104,106,165,221,225,236',
       }
     }
 
-    if(!is.character(timeStamp) && timeStamp) {
-      timeStamp <- "%Y%m%d %H:%M:%OS"
-    } else {
-      timeStamp <- NULL
-    }
-    # set up default event handlers, if
-    # callback is not set
-    if(missing(CALLBACK)) {
-      if(missing(eventTickPrice))
-        eventTickPrice   <- e_tick_price
-      if(missing(eventTickSize))
-        eventTickSize    <- e_tick_size
-      if(missing(eventTickOption)) 
-        eventTickOption  <- e_tick_option
-      if(missing(eventTickGeneric)) 
-        eventTickGeneric <- e_tick_generic
-      if(missing(eventTickString)) 
-        eventTickString  <- e_tick_string
-      if(missing(eventTickEFP))
-        eventTickEFP     <- e_tick_EFP
-    } 
-    else if(is.null(CALLBACK)) {
-        eventTickPrice   <- NULL
-        eventTickSize    <- NULL
-        eventTickOption  <- NULL
-        eventTickGeneric <- NULL
-        eventTickString  <- NULL
-        eventTickEFP     <- NULL
-    }
-
+    if(is.null(CALLBACK))
+      CALLBACK <- twsDEBUG
 
     snapshot <- ifelse(snapshot,"1","0")
 
@@ -81,7 +51,7 @@ function (conn, Contract, tickGenerics='100,101,104,106,165,221,225,236',
       for(n in 1:length(Contract)) {
         signals <- c(.twsOutgoingMSG$REQ_MKT_DATA,
                      VERSION,
-                     ticker_id,
+                     ticker_id,  # why?
                      Contract[[n]]$symbol,
                      Contract[[n]]$sectype,
                      Contract[[n]]$expiry,
@@ -95,18 +65,15 @@ function (conn, Contract, tickGenerics='100,101,104,106,165,221,225,236',
                      tickGenerics,
                      snapshot)
     
-         writeBin(signals, con) 
-    #    for (i in 1:length(signals)) {
-    #        writeBin(signals[i], con)
-    #    }
+        writeBin(signals, con) 
         ticker_id <- as.character(as.numeric(tickerId)+n)
       }
-      msg_expected_length <- NA
-    } else {
+      #msg_expected_length <- NA
+    } #else {
       # reading from a file
-      msg_expected_length <- as.numeric(readBin(con,character(), 1))
+      #msg_expected_length <- as.numeric(readBin(con,character(), 1))
       #timeStamp <- NULL #disable erroneous R timestamps
-    }
+    #}
 
     if(!missing(CALLBACK) && is.na(list(CALLBACK))) {
       if(inherits(conn, 'twsPlayback')) {
@@ -119,107 +86,8 @@ function (conn, Contract, tickGenerics='100,101,104,106,165,221,225,236',
 
     waiting <- TRUE
 
-    msg_length <- ifelse(inherits(conn, 'twsPlayback'), 3, 1)
-    msg_position <- 0 # where we are in the message - only relevant for playback
-    sys.time <- NULL # used for timeStamp interpretation
+    CALLBACK(conn, eWrapper=eventWrapper, timestamp=timeStamp, file=file, playback=playback, ...)
 
-#used for vignette counting only --- not in production code
-#VCOUNT <- 0
-
-    if(missing(CALLBACK) || is.null(CALLBACK)) {
-      while (waiting) {
-
-        # read the msg header for each new message,
-        # for standard connection to TWS this is the first
-        # character string received. For playback the
-        # actual message will be the 3rd (after the date and time
-        # stamps).  
-
-        curMsg <- readBin(con, character(), msg_length)
-
-        if(!is.null(timeStamp)) {
-          if(msg_length > 1) {
-            last.time <- sys.time
-            sys.time <- as.POSIXct(paste(curMsg[1:2],collapse=' '))
-            if(!is.null(last.time)) {
-              Sys.sleep((sys.time-last.time)*playback)
-            } 
-          } else sys.time <- Sys.time()
-        } else sys.time <- NULL
-
-        curMsg <- curMsg[msg_length] 
-
-        msg_position <- msg_position + msg_length 
-
-        if (length(curMsg) > 0) {
-          if (curMsg == .twsIncomingMSG$ERR_MSG) {
-              if (!errorHandler(con, verbose, OK = c(165, 300, 366, 2104,2106,2107))) {
-                cat("\n")
-                stop("Unable to complete market data request")
-              }
-              msg_position <- msg_position + 4
-          }
-          if (curMsg == .twsIncomingMSG$TICK_PRICE) {
-              contents <- readBin(con, character(), 6)
-              if(is.null(eventTickPrice)) {
-                if (!is.null(timeStamp)) cat(as.character(sys.time),' ',file=file,append=TRUE)
-                cat(curMsg,paste(contents),'\n',file=file, append=TRUE)
-              } else eventTickPrice(curMsg,contents,sys.time,file)
-              msg_position <- msg_position + 6
-          }
-          if (curMsg == .twsIncomingMSG$TICK_SIZE) {
-              contents <- readBin(con, character(), 4)
-              if(is.null(eventTickSize)) {
-                if (!is.null(timeStamp)) cat(as.character(sys.time),' ',file=file,append=TRUE)
-                cat(curMsg,paste(contents),'\n',file=file, append=TRUE)
-              } else eventTickSize(curMsg,contents,sys.time,file)
-              msg_position <- msg_position + 4
-          }
-          if (curMsg == .twsIncomingMSG$TICK_OPTION) {
-              contents <- readBin(con, character(), 5)
-              if(is.null(eventTickOption)) {
-                if (!is.null(timeStamp)) cat(as.character(sys.time),' ',file=file,append=TRUE)
-                cat(curMsg,paste(contents),'\n',file=file, append=TRUE)
-              } else eventTickOption(curMsg,contents,sys.time,file)
-              msg_position <- msg_position + 5
-          }
-          if (curMsg == .twsIncomingMSG$TICK_GENERIC) {
-              contents <- readBin(con, character(), 4)
-              if(is.null(eventTickGeneric)) {
-                if (!is.null(timeStamp)) cat(as.character(sys.time),' ',file=file,append=TRUE)
-                cat(curMsg,paste(contents),'\n',file=file, append=TRUE)
-              } else eventTickGeneric(curMsg,contents,sys.time,file)
-              msg_position <- msg_position + 4
-          }
-          if (curMsg == .twsIncomingMSG$TICK_STRING) {
-              contents <- readBin(con, character(), 4)
-              if(is.null(eventTickString)) {
-                if (!is.null(timeStamp)) cat(as.character(sys.time),' ',file=file,append=TRUE)
-                cat(curMsg,paste(contents),'\n',file=file, append=TRUE)
-              } else eventTickString(curMsg,contents,sys.time,file)
-              msg_position <- msg_position + 4
-              if(snapshot == '1') 
-                waiting <- FALSE
-          }
-          if (curMsg == .twsIncomingMSG$TICK_EFP) {
-              contents <- readBin(con, character(), 13)
-              if(is.null(eventTickEFP)) {
-                if (!is.null(timeStamp)) cat(as.character(sys.time),' ',file=file,append=TRUE)
-                cat(curMsg,paste(contents),'\n',file=file, append=TRUE)
-              } else {
-                cat('<efp> ')
-                cat(curMsg,paste(contents),'\n')
-              }
-              msg_position <- msg_position + 13
-          }
-          flush.console()
-          if(!is.na(msg_expected_length) && msg_position == msg_expected_length)
-            waiting <- FALSE
-        }
-#VCOUNT <- VCOUNT + 1
-#if(VCOUNT == 20) waiting <- FALSE
-      }
-    } else CALLBACK(con,...)
 }
 
 `cancelMktData` <- function(conn,tickerId) {
